#include "project.h"
#include "stdio.h"
#include "tty_utils.h"

char    cmd[STR_BUF_LEN+1] ;

void init_hardware(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    tty_init() ;
    PWM_Start() ;
}

void help(void)
{
    print("=== Usage ===\n\r") ;
    print("period  <value> : set PWM period to  <value>\n\r") ;
    print("compare <value> : set PWM compare to <value>\n\r") ;
    print("status          : print PWM status\n\r") ;
    print("help            : print this usage\n\r") ;
    print("=============\r\n") ;
}

void status_report(void)
{
    uint16_t period, compare ;
    period = PWM_ReadPeriod() ;
    compare = PWM_ReadCompare() ;
    snprintf(str, STR_BUF_LEN,
        "PWM: Period: %hu,  Compare: %hu\n\r",
        period, compare) ;
    print(str) ;
}

void set_pwm_period(uint16_t period)
{
    PWM_Stop() ;
    PWM_WritePeriod(period) ;
    PWM_WriteCounter(0) ;
    PWM_Enable() ;
    status_report() ;
}

void set_pwm_compare(uint16_t compare) 
{
    // PWM_Stop() // for compare PWM can be running
    PWM_WriteCompare(compare) ;
    PWM_WriteCounter(0) ; // When counter == 0 compare value gets updated
    // PWM_Enable() // for compare PWM can be running
    status_report() ;
}

int main(void)
{
    uint16_t period, compare ;
    
    init_hardware() ;
    
    period = PWM_ReadPeriod() ;
    compare = PWM_ReadCompare() ;
    
    splash("PWM TEST") ;
    help() ;
    status_report() ;
    
    prompt() ;

    for(;;) {
        if (get_line()) {
            sscanf(str, "%s", cmd) ;
            switch(cmd[0]) {
            case 'p': case 'P': // period
                sscanf(str, "%*s %hu", &period)  ;
                set_pwm_period(period) ;
                break ;
            case 'c': case 'C': // compare
                sscanf(str, "%*s %hu", &compare) ;
                set_pwm_compare(compare) ;
                break ;
            case 's': case 'S': // status report
                status_report() ;
                break ;
            default: 
                help() ;
                break ;
            }
            prompt() ;
        }
    }
}

